#!/usr/bin/ruby

require 'fork_manager'
require 'optparse'

sourcedir = ""
destdir = ""
oggquality = 0
numthreads = 2
createsubdir = nil

OptionParser.new do |opts|

  opts.on("-s", "--src DIR", String, "Specify the source directory") do |dir|
    sourcedir = dir
  end
  
  opts.on("-d", "--dest DIR", String, "Specify the destination directory") do |dir|
    destdir = dir
  end

  opts.on("-q", "--quality N", Float, "Convert to OGG with this quality") do |q|
    oggquality = q
  end

  opts.on("-t", "--threads N", Integer, "Use this number of threads (default 2)") do |t|
    numthreads = t
  end

  opts.on("-c", "--create", "Create same subdirectory at destination") do |c|
    createsubdir = c
  end

  opts.on("-h", "--help", "Show this message") do
    puts opts
    exit
  end

end.parse!

puts "SOURCE #{sourcedir}; DEST #{destdir}; OGGQ #{oggquality}; THREADS #{numthreads}"

flaclist = Dir.glob("#{sourcedir}/*.flac")

if createsubdir
  pathcomps = sourcedir.split(/\//)
  if pathcomps.length > 1
    # puts pathcomps[pathcomps.length-1]
    destdir += "/"
    destdir += pathcomps[pathcomps.length-1]
    if File.exists?(destdir) == false
      Dir.mkdir(destdir)
    end
  end
end

if oggquality == 0
  flaclist.each do |file|
    system("cp -p -v \"#{file}\" \"#{destdir}\"/")
  end
else
  fm = ForkManager.new(numthreads)
  fm.manage do
    flaclist.each do |file|
      fm.fork do
        # Make sure destfile is a new string object and not an alias for file!
        destfile = ""
        destfile += file
        destfile[sourcedir] = destdir
        destfile[".flac"] = ".ogg"
        puts "#{file} -> #{destfile}"
        system("oggenc -Q -q #{oggquality} -o \"#{destfile}\" \"#{file}\"")
      end
    end
  end
end
